// rollup.config.js
import { createRequire } from "module";
const require = createRequire(import.meta.url);

import svelte from 'rollup-plugin-svelte';
import resolve from '@rollup/plugin-node-resolve';
import commonjs from '@rollup/plugin-commonjs';
import postcss from 'rollup-plugin-postcss';


// dependencies
const fs = require("fs")
const postcssglobal = require("postcss")
const atImport = require("postcss-import")

// css to be processed
const globalStyles = fs.readFileSync("src/styles/global.css", "utf8")

// Read the content of the existing CSS file

export default {
  input: 'src/index.js',
  output: {
    file: 'dist/index.js',
    format: 'esm',
  },
  plugins: [
    svelte({
      // Configuration for Svelte plugin
    }),
    resolve(),
    commonjs(),
    postcssglobal()
      .use(atImport())
      .process(globalStyles, {
        // `from` option is needed here
        from: "src/styles/global.css"
      })
      .then((result) => {
        const output = result.css
        fs.writeFileSync("dist/spectrum-ui-components.css", result.css)
      }),
    postcss({
      minimize: true,
      // Extract CSS into a separate file
      extract: false,
    })
  
    ],
};
