import fs from 'fs';
const mime = require('mime-types');

import { writable } from 'svelte/store';


export const sortFoldersFirst = (a, b)=>{
    let aValue = a.type === "folder" ? 0 : 1;
    let bValue = b.type === "folder" ? 0 : 1;
    return aValue - bValue;
};
export const sortByName = (a, b) => {
    var nameA = a.name.toUpperCase(); // ignore upper and lowercase
    var nameB = b.name.toUpperCase(); // ignore upper and lowercase
    if (nameA < nameB) return -1;
    if (nameA > nameB) return 1;
    // names must be equal
    return 0;
}
export const updateFolders = writable(Date.now());
/*
export var _getAllFilesFromFolder = function(dir) {
    var results = [];
    fs.readdirSync(dir).forEach(function(file) {
        try{
            file = dir+'/'+file;
            var stat = fs.lstatSync(file);

            if (stat && stat.isDirectory()) {
                results = results.concat(_getAllFilesFromFolder(file))
            } else results.push(file);
        }catch(error){
            return;
        }
    });
    return results;
};
*/
export class FileTree {
	constructor(path){
		this.exists = fs.existsSync(path);
		this.path = path;
	  	this.name = this.path.split('/')[this.path.split('/').length -1];
		this.children = [];
        this.childrenStore = writable(this.children);

		if(this.exists){
			this.type = fs.lstatSync(path).isDirectory() ? "folder" : "file";
		}
		if(this.type === "file"){
            this.mime = mime.lookup(this.path);
			this.extension = this.getExtension();
		}

	}
	existing(){this.exists = fs.existsSync(this.path)}
    hasReadAccess(){
        try{
            fs.accessSync(this.path, fs.constants.R_OK)
            return true;
        }catch(error){ 
            //console.error(error);
            return false;
        }
    }
    hasWriteAccess(){
        try{
            fs.accessSync(this.path, fs.constants.W_OK)
            return true;
        }catch(error){
            //console.error(error);
            return false;
        }
    }
    getGrandchildren(isGrandChild = false){
        if(!isGrandChild) this.refreshFolder(); 
        window.requestIdleCallback(()=>{
            this.children = this.getChildren();
            this.children.forEach( f => f.getGrandchildren(true));
            this.setChildren(this.children); 
        });
    }
    hasExtension(name = this.name){
        return name.match(/\.([0-9a-z]+)$/i)
    }
    getExtension(name = this.name){
        const extension = this.hasExtension(name);
        return extension ? extension[1] : "";
    }

    setChildren(fileArray){
        this.childrenStore.set(fileArray); 
        updateFolders.set(Date.now());
    }
    refreshFolder(){
        this.children = this.getChildren().map( f => this.children.find(e=> e.path === f.path) || f);
        this.setChildren(this.children);
    }
	getChildren(){ 
        try{
            if(this.type === "folder" && this.hasReadAccess()){
                return fs.readdirSync(this.path).map((name) => {
                    let filePath = this.path + "/" + name;
                    let isFile = !fs.lstatSync(filePath).isDirectory();
                    let creator = fileTypes[this.getExtension(name)] || FileTree;
                    return new creator(filePath);
                });
            }
            return this.children;
        }catch(error){
           // console.error(error);
            return this.children;
        }
	}
    async traverse(callback){
        callback(this);   
        this.children.forEach(child => child.traverse(callback))   
    } 
} 

export class Image extends FileTree {
    constructor(path) {
        super(path, Image);
        this.thumbnail = writable("");
        this.getThumbnail();        
    }
    getThumbnail(imagePath = this.path){
        var fs = require('fs'); 
        var mime = 'image/' + this.getExtension(imagePath); 
        var encoding = 'base64'; 
        var data = fs.existsSync(imagePath) ? fs.readFileSync(imagePath).toString(encoding) : "";
        this.thumbnail.set('data:' + mime + ';' + encoding + ',' + data);
    }
}
export class Preset extends Image { 
    constructor(path) {
        super(path, Preset);
        this.getThumbnail(this.path.replace('.ffx', '.png'));        
    }
}

const fileTypes = {
    "ffx" : Preset,
    "png" : Image
}
